# -*- coding: utf-8 -*-
from django.contrib import admin

from django import forms
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy
from django.utils.six import with_metaclass

from job_offers.models import Application, JobOffer, RelatedObject


@admin.register(RelatedObject)
class RelatedObjectAdmin(admin.ModelAdmin):
    """Contract type for Job offers"""
    list_display = [
        'name',
        'type',
        'slug',
    ]
    list_filter = [
        'type',
    ]


class RelatedObjectFields(type(forms.ModelForm)):
    def __new__(self, name, bases, attrs):
        for slug, related_name in RelatedObject._meta.get_field('type').choices:
            attrs[slug] = forms.ModelChoiceField(
                label=related_name,
                queryset=RelatedObject.objects.filter(type=slug),
                to_field_name='slug',
                required=False
            )
        return super(RelatedObjectFields, self).__new__(self, name, bases, attrs)


class JobOfferForm(with_metaclass(RelatedObjectFields, forms.ModelForm)):
    def __init__(self, *args, **kw):
        if 'instance' in kw:
            initial = dict(kw['instance']._relations)
            initial.update(kw.get('initial') or [])
            kw['initial'] = initial
        super(JobOfferForm, self).__init__(*args, **kw)

    class Meta:
        fields = [
            'title',
            'slug',
            'short_description',
            'job_description',
            'competences',
            'reference',
            'date',
            'weight',
            'is_active',
            'tags',
        ]

    def save(self, **kw):
        instance = super(JobOfferForm, self).save(**kw)
        for slug, related_name in RelatedObject._meta.get_field('type').choices:
            setattr(instance, slug, self.cleaned_data[slug])
        return instance


@admin.register(JobOffer)
class JobOfferAdmin(admin.ModelAdmin):
    """Job offers"""
    form = JobOfferForm

    search_fields = [
        'title',
        'reference',
    ]
    list_filter = [
    ]
    list_display = [
        'display',
    ]
    prepopulated_fields = {
        'slug': ['title']
    }

    def display(self, job_offer):
        return str(job_offer)


class ApplicationModelForm(forms.ModelForm):
    """Forms for Application"""
    message = forms.CharField(
        widget=forms.Textarea
    )

    class Meta:
        model = Application
        fields = [
            'job_offer',
            'message',
            'last_name',
            'first_name',
            'email',
            'phone',
            'cv',
            'cover_letter',
            'other_document',
        ]


@admin.register(Application)
class ApplicationAdmin(admin.ModelAdmin):
    """Job application"""
    form = ApplicationModelForm

    search_fields = [
        'last_name',
        'email',
        'phone',
    ]
    list_filter = [
        'date',
    ]
    list_display = [
        'display',
        'last_name',
        'email',
        'job_location',
        'job_area',
        'get_cv',
    ]

    def display(self, application):
        if application.job_offer is None:
            return ugettext_lazy('Speculative application')
        return str(application.job_offer)

    def job_area(self, obj):
        if obj.job_offer is None:
            return ''
        return obj.job_offer.geo_area

    def job_location(self, obj):
        if obj.job_offer is None:
            return ''
        return obj.job_offer.location

    def get_cv(self, obj):
        return format_html('<a href="{}">{}</a>', obj.cv.url, obj.last_name + ' cv')

    get_cv.short_description = "Cv"
