# -*- coding: utf-8 -*-


from django import forms
from django.db.models import Q
from django.utils.translation import ugettext_lazy
from django.utils.six import with_metaclass

from job_offers.models import Application, RelatedObject


class ApplicationForm(forms.ModelForm):
    """Form for job application"""

    class Meta:
        model = Application
        fields = [
            'message',
            'last_name',
            'first_name',
            'email',
            'phone',
            'cv',
            'cover_letter',
            'other_document',
        ]


class RelatedObjectFields(type(forms.Form)):
    def __new__(self, name, bases, attrs):
        for slug, related_name in RelatedObject._meta.get_field('type').choices:
            attrs[slug] = forms.ModelChoiceField(
                label=related_name,
                queryset=RelatedObject.objects.filter(type=slug),
                to_field_name='slug',
                required=False
            )
        return super(RelatedObjectFields, self).__new__(self, name, bases, attrs)


class SearchForm(with_metaclass(RelatedObjectFields, forms.Form)):
    keywords = forms.CharField(
        label=ugettext_lazy('Keywords'),
        max_length=100,
        required=False
    )
    order = forms.ChoiceField(
        choices=RelatedObject._meta.get_field('type').choices,
        required=False,
    )
    reverse = forms.BooleanField(
        label=ugettext_lazy('Reverse order'),
        required=False,
    )

    def clean_keywords(self):
        return self.cleaned_data['keywords'].split()

    def get_keyword_predicate(self, word):
        return (Q(title__contains=word) |
                Q(job_description__contains=word) |
                Q(reference__contains=word) |
                Q(tags__contains=word) |
                Q(competences__contains=word))

    def get_predicates(self):
        predicate = Q()
        if self.cleaned_data.get('keywords'):
            predicate &= self.get_keyword_predicate()

        for related_object in self.cleaned_data.values():
            if not isinstance(related_object, RelatedObject):
                continue
            predicate &= Q(relations=related_object)
        return predicate

    def get_order_by(self):
        if not self.cleaned_data.get('order'):
            return 'weight'

        return '{}{}'.format(
            '-' if self.predicates.get('reverse') else '',
            self.cleaned_data['order'],
        )
